@@ -164,60 +164,15 @@ def filter_leaf(domain, field, model):
 
 def prepare_reference_domain(domain, reference):
     "convert domain to replace reference fields by their local part"
-
-    def value2reference(value):
-        model, ref_id = None, None
-        if isinstance(value, str) and ',' in value:
-            model, ref_id = value.split(',', 1)
-            if ref_id != '%':
-                try:
-                    ref_id = int(ref_id)
-                except ValueError:
-                    model, ref_id = None, value
-        elif (isinstance(value, (list, tuple))
-                and len(value) == 2
-                and isinstance(value[0], str)
-                and (isinstance(value[1], int) or value[1] == '%')):
-            model, ref_id = value
-        else:
-            ref_id = value
-        return model, ref_id
-
     if domain in ('AND', 'OR'):
         return domain
     elif is_leaf(domain):
-        if domain[0] == reference:
-            if domain[1] in {'=', '!='}:
-                model, ref_id = value2reference(domain[2])
-                if model is not None:
-                    if ref_id == '%':
-                        if domain[1] == '=':
-                            return [reference + '.id', '!=', None, model]
-                        else:
-                            return [reference, 'not like', domain[2]]
-                    return [reference + '.id', domain[1], ref_id, model]
-            elif domain[1] in {'in', 'not in'}:
-                model_values = {}
-                for value in domain[2]:
-                    model, ref_id = value2reference(value)
-                    if model is None:
-                        break
-                    model_values.setdefault(model, []).append(ref_id)
-                else:
-                    new_domain = ['OR'] if domain[1] == 'in' else ['AND']
-                    for model, ref_ids in model_values.items():
-                        if '%' in ref_ids:
-                            if domain[1] == 'in':
-                                new_domain.append(
-                                    [reference + '.id', '!=', None, model])
-                            else:
-                                new_domain.append(
-                                    [reference, 'not like', model + ',%'])
-                        else:
-                            new_domain.append(
-                                [reference + '.id', domain[1], ref_ids, model])
-                    return new_domain
-            return []
+        # When a Reference field is using the dotted notation the model
+        # specified must be removed from the clause
+        if domain[0].count('.') and len(domain) > 3:
+            local_name, target_name = domain[0].split('.', 1)
+            if local_name == reference:
+                return [target_name] + list(domain[1:3] + domain[4:])
         return domain
     else:
         return [prepare_reference_domain(d, reference) for d in domain]
@@ -326,12 +281,18 @@ def concat(*domains, **kwargs):
 def unique_value(domain):
     "Return if unique, the field and the value"
     if (isinstance(domain, list)
-            and len(domain) == 1
-            and '.' not in domain[0][0]
-            and domain[0][1] == '='):
-        return True, domain[0][1], domain[0][2]
-    else:
-        return False, None, None
+            and len(domain) == 1):
+        domain, = domain
+        name = domain[0]
+        value = domain[2]
+        count = 0
+        if len(domain) == 4 and name[-3:] == '.id':
+            count = 1
+            model = domain[3]
+            value = [model, value]
+        if name.count('.') == count and domain[1] == '=':
+            return True, domain[1], value
+    return False, None, None
 
 
 def parse(domain):
@@ -379,8 +340,6 @@ class And(object):
             if isinstance(part, And):
                 part_inversion = part.inverse(symbol, context)
                 evaluated = isinstance(part_inversion, bool)
-                if symbol not in part.variables:
-                    continue
                 if not evaluated:
                     result.append(part_inversion)
                 elif part_inversion:
@@ -411,8 +370,9 @@ class Or(And):
     def inverse(self, symbol, context):
         result = []
         known_variables = set(context.keys())
-        if not known_variables >= (self.variables - {symbol}):
-            # In this case we don't know enough about this OR part, we
+        if (symbol not in self.variables
+                and not known_variables >= self.variables):
+            # In this case we don't know anything about this OR part, we
             # consider it to be True (because people will have the constraint
             # on this part later).
             return True
@@ -632,6 +592,10 @@ def test_unique_value():
     assert unique_value(domain)[0] is False
     domain = [['a.b', '=', 1]]
     assert unique_value(domain)[0] is False
+    domain = [['a.id', '=', 1, 'model']]
+    assert unique_value(domain) == (True, '=', ['model', 1])
+    domain = [['a.b.id', '=', 1, 'model']]
+    assert unique_value(domain) == (False, None, None)
 
 
 def test_evaldomain():
@@ -849,6 +813,7 @@ if __name__ == '__main__':
     test_oror_inversion()
     test_parse()
     test_simplify()
+    test_unique_value()
     test_evaldomain()
     test_localize()
     test_prepare_reference_domain()
