@@ -10,8 +10,7 @@ import datetime
 import io
 from collections import OrderedDict
 
-from tryton.common import (
-    untimezoned_date, timezoned_date, datetime_strftime, date_format)
+from tryton.common import untimezoned_date, timezoned_date
 from tryton.common.datetime_ import date_parse
 from tryton.common.timedelta import parse as timedelta_parse
 from tryton.common.timedelta import format as timedelta_format
@@ -107,34 +106,17 @@ def test_group_operator():
     assert list(group_operator(iter(['a', '>', '=', '=']))) == ['a', '>=', '=']
 
 
-def likify(value, escape='\\'):
+def likify(value):
     "Add % if needed"
     if not value:
         return '%'
-    escaped = value.replace(escape + '%', '').replace(escape + '_', '')
-    if '%' in escaped or '_' in escaped:
+    escaped = value.replace('%%', '__')
+    if '%' in escaped:
         return value
     else:
         return '%' + value + '%'
 
 
-def is_full_text(value, escape='\\'):
-    escaped = value.strip('%')
-    escaped = escaped.replace(escape + '%', '').replace(escape + '_', '')
-    if '%' in escaped or '_' in escaped:
-        return False
-    return value.startswith('%') and value.endswith('%')
-
-
-def is_like(value, escape='\\'):
-    escaped = value.replace(escape + '%', '').replace(escape + '_', '')
-    return '%' in escaped or '_' in escaped
-
-
-def unescape(value, escape='\\'):
-    return value.replace(escape + '%', '%').replace(escape + '_', '_')
-
-
 def quote(value):
     "Quote string if needed"
     if not isinstance(value, str):
@@ -193,7 +175,7 @@ def append_ending_clause(domain, clause, deep):
 def default_operator(field):
     "Return default operator for field"
     if field['type'] in ('char', 'text', 'many2one', 'many2many', 'one2many',
-            'reference', 'one2one'):
+            'reference'):
         return 'ilike'
     else:
         return '='
@@ -289,8 +271,7 @@ def convert_value(field, value, context=None):
     def convert_datetime():
         if not value:
             return
-        format_ = (
-            date_format(context.get('date_format')) + ' ' + time_format(field))
+        format_ = context.get('date_format', '%x') + ' %X'
         try:
             dt = date_parse(value, format_)
             return untimezoned_date(dt)
@@ -300,7 +281,7 @@ def convert_value(field, value, context=None):
     def convert_date():
         if not value:
             return
-        format_ = date_format(context.get('date_format'))
+        format_ = context.get('date_format', '%x')
         try:
             return date_parse(value, format_).date()
         except (ValueError, TypeError):
@@ -519,26 +500,19 @@ def format_value(field, value, target=None, context=None):
                 and value is not 0
                 and not isinstance(value, (float, Decimal))):
             return ''
-        digit = 0
         if isinstance(value, Decimal):
             cast = Decimal
         else:
             cast = float
         factor = cast(field.get('factor', 1))
-        string_ = str(value * factor)
-        if 'e' in string_:
-            string_, exp = string_.split('e')
-            digit -= int(exp)
-        if '.' in string_:
-            digit += len(string_.rstrip('0').split('.')[1])
-        return locale.localize(
-            '{0:.{1}f}'.format(value * factor or 0, digit), True)
+        try:
+            digit = len(str(value * factor).rstrip('0').split('.')[1])
+        except IndexError:
+            digit = 0
+        return locale.format('%.*f', (digit, value * factor or 0), True)
 
     def format_selection():
-        if isinstance(field['selection'], (tuple, list)):
-            selections = dict(field['selection'])
-        else:
-            selections = {}
+        selections = dict(field['selection'])
         return selections.get(value, value) or ''
 
     def format_reference():
@@ -550,20 +524,20 @@ def format_value(field, value, target=None, context=None):
     def format_datetime():
         if not value:
             return ''
+        format_ = context.get('date_format', '%x') + ' ' + time_format(field)
         if not isinstance(value, datetime.datetime):
             time = datetime.datetime.combine(value, datetime.time.min)
         else:
             time = timezoned_date(value)
-        format_ = date_format(context.get('date_format'))
-        if time.time() != datetime.time.min:
-            format_ += ' ' + time_format(field)
-        return datetime_strftime(time, format_)
+        if time.time() == datetime.time.min:
+            format_ = '%x'
+        return time.strftime(format_)
 
     def format_date():
         if not value:
             return ''
-        format_ = date_format(context.get('date_format'))
-        return datetime_strftime(value, format_)
+        format_ = context.get('date_format', '%x')
+        return value.strftime(format_)
 
     def format_time():
         if not value:
@@ -1043,7 +1017,8 @@ class DomainParser(object):
             if name.endswith('.rec_name'):
                 name = name[:-9]
             if name not in self.fields:
-                if is_full_text(value):
+                escaped = value.replace('%%', '__')
+                if escaped.startswith('%') and escaped.endswith('%'):
                     value = value[1:-1]
                 return quote(value)
             field = self.fields[name]
@@ -1054,14 +1029,15 @@ class DomainParser(object):
                 target = None
 
             if 'ilike' in operator:
-                if is_full_text(value):
+                escaped = value.replace('%%', '__')
+                if escaped.startswith('%') and escaped.endswith('%'):
                     value = value[1:-1]
-                elif not is_like(value):
+                elif '%' not in escaped:
                     if operator == 'ilike':
                         operator = '='
                     else:
                         operator = '!'
-                    value = unescape(value)
+                    value = value.replace('%%', '%')
             def_operator = default_operator(field)
             if def_operator == operator.strip():
                 operator = ''
@@ -1275,7 +1251,7 @@ class DomainParser(object):
                         operator = negate_operator(default_operator(field))
                     if field['type'] in ('integer', 'float', 'numeric',
                             'datetime', 'date', 'time'):
-                        if isinstance(value, str) and '..' in value:
+                        if value and '..' in value:
                             lvalue, rvalue = value.split('..', 1)
                             lvalue = convert_value(field, lvalue, self.context)
                             rvalue = convert_value(field, rvalue, self.context)
@@ -1377,7 +1353,7 @@ def test_string():
     assert dom.string([('name', 'ilike', '%<%')]) == 'Name: "" "<"'
     assert dom.string([('name', 'ilike', 'Doe')]) == 'Name: =Doe'
     assert dom.string([('name', 'ilike', 'Doe%')]) == 'Name: Doe%'
-    assert dom.string([('name', 'ilike', 'Doe\\%')]) == 'Name: =Doe%'
+    assert dom.string([('name', 'ilike', 'Doe%%')]) == 'Name: =Doe%'
     assert dom.string([('name', 'not ilike', '%Doe%')]) == 'Name: !Doe'
     assert dom.string([('name', 'in', ['John', 'Jane'])]) == 'Name: John;Jane'
     assert dom.string([('name', 'not in', ['John', 'Jane'])]) == \
