@@ -1,11 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 "Form"
+import csv
+import datetime
 import gettext
-import gtk
-import gobject
+import locale
+import os
+import tempfile
 
-from gi.repository import Gtk
+from gi.repository import Gdk, GLib, Gtk
 
 from tryton.gui.window.view_form.screen import Screen
 from tryton.action import Action
@@ -20,7 +23,7 @@ from tryton.signal_event import SignalEvent
 from tryton.common import message, sur, sur_3b, timezoned_date
 import tryton.common as common
 from tryton.common import RPCExecute, RPCException
-from tryton.common.datetime_strftime import datetime_strftime
+from tryton.common.underline import set_underline
 from tryton import plugins
 
 from .tabcontent import TabContent
@@ -53,11 +56,11 @@ class Form(SignalEvent, TabContent):
             self._record_message)
 
         self.screen.signal_connect(self, 'record-modified',
-            lambda *a: gobject.idle_add(self._record_modified, *a))
+            lambda *a: GLib.idle_add(self._record_modified, *a))
         self.screen.signal_connect(self, 'record-saved', self._record_saved)
-        self.screen.signal_connect(self, 'attachment-count',
-                self._attachment_count)
-        self.screen.signal_connect(self, 'unread-note', self._unread_note)
+        self.screen.signal_connect(
+            self, 'resources',
+            lambda screen, resources: self.update_resources(resources))
 
         if res_id not in (None, False):
             if isinstance(res_id, int):
@@ -88,15 +91,13 @@ class Form(SignalEvent, TabContent):
         return (self.model == model
             and self.res_id == attributes.get('res_id')
             and self.attributes.get('domain') == attributes.get('domain')
+            and (self.attributes.get('mode') or []) == (
+                attributes.get('mode') or [])
             and self.attributes.get('view_ids') == attributes.get('view_ids')
-            and (attributes.get('view_ids')
-                or (self.attributes.get('mode') or ['tree', 'form']) == (
-                    attributes.get('mode') or ['tree', 'form']))
-            and self.screen.local_context == attributes.get('context')
+            and self.attributes.get('context') == attributes.get('context')
+            and self.attributes.get('limit') == attributes.get('limit')
             and self.attributes.get('search_value') == (
-                attributes.get('search_value'))
-            and self.attributes.get('tab_domain') == (
-                attributes.get('tab_domain')))
+                attributes.get('search_value')))
 
     def __hash__(self):
         return id(self)
@@ -109,7 +110,7 @@ class Form(SignalEvent, TabContent):
     def sig_attach(self, widget=None):
         def window(widget):
             return Attachment(
-                record, lambda: self.update_attachment_count(reload=True))
+                record, lambda: self.refresh_resources(reload=True))
 
         def add_file(widget):
             filenames = common.file_selection(_("Select"), multi=True)
@@ -129,74 +130,62 @@ class Form(SignalEvent, TabContent):
         record = self.screen.current_record
         menu = button._menu = Gtk.Menu()
         for name, callback in Attachment.get_attachments(record):
-            item = Gtk.MenuItem()
-            item.set_label(name)
+            item = Gtk.MenuItem(label=name)
             item.connect('activate', activate, callback)
             menu.add(item)
         menu.add(Gtk.SeparatorMenuItem())
-        add_item = Gtk.MenuItem()
-        add_item.set_label(_("Add..."))
+        add_item = Gtk.MenuItem(label=_("Add..."))
         add_item.connect('activate', add_file)
         menu.add(add_item)
-        manage_item = Gtk.MenuItem()
-        manage_item.set_label(_("Manage..."))
+        manage_item = Gtk.MenuItem(label=_("Manage..."))
         manage_item.connect('activate', window)
         menu.add(manage_item)
         menu.show_all()
         menu.connect('deactivate', self._popup_menu_hide, button)
         self.action_popup(button)
 
-    def update_attachment_count(self, reload=False):
-        record = self.screen.current_record
-        if record:
-            attachment_count = record.get_attachment_count(reload=reload)
-        else:
-            attachment_count = 0
-        self._attachment_count(None, attachment_count)
-
-    def _attachment_count(self, widget, signal_data):
-        label = _('Attachment(%d)') % signal_data
-        self.buttons['attach'].set_label(label)
-        if signal_data:
-            # FIXME
-            icon = 'tryton-attach'
-        else:
-            icon = 'tryton-attach'
-        image = common.IconFactory.get_image(icon, gtk.ICON_SIZE_LARGE_TOOLBAR)
-        image.show()
-        self.buttons['attach'].set_icon_widget(image)
-        record = self.screen.current_record
-        self.buttons['attach'].props.sensitive = bool(
-            record.id >= 0 if record else False)
-
     def sig_note(self, widget=None):
         record = self.screen.current_record
         if not record or record.id < 0:
             return
         Note(record,
-            lambda: self.update_unread_note(reload=True))
+            lambda: self.refresh_resources(reload=True))
 
-    def update_unread_note(self, reload=False):
+    def refresh_resources(self, reload=False):
         record = self.screen.current_record
-        if record:
-            unread = record.get_unread_note(reload=reload)
-        else:
-            unread = 0
-        self._unread_note(None, unread)
+        self.update_resources(
+            record.get_resources(reload=reload) if record else None)
 
-    def _unread_note(self, widget, signal_data):
-        label = _('Note(%d)') % signal_data
-        self.buttons['note'].set_label(label)
-        if signal_data:
-            self.buttons['note'].set_stock_id('tryton-note-hi')
-        else:
-            self.buttons['note'].set_stock_id('tryton-note')
+    def update_resources(self, resources):
+        if not resources:
+            resources = {}
         record = self.screen.current_record
-        if not record or record.id < 0:
-            sensitive = False
+        sensitive = record.id >= 0 if record else False
+
+        def update(name, label, icon, badge):
+            button = self.buttons[name]
+            button.set_label(label)
+            image = common.IconFactory.get_image(
+                icon, Gtk.IconSize.LARGE_TOOLBAR, badge=badge)
+            image.show()
+            button.set_icon_widget(image)
+            button.props.sensitive = sensitive
+
+        attachment_count = resources.get('attachment_count', 0)
+        badge = 1 if attachment_count else None
+        label = _("Attachment (%s)") % attachment_count
+        update('attach', label, 'tryton-attach', badge)
+
+        note_count = resources.get('note_count', 0)
+        note_unread = resources.get('note_unread', 0)
+        if note_unread:
+            badge = 2
+        elif note_count:
+            badge = 1
         else:
-            sensitive = True
-        self.buttons['note'].props.sensitive = sensitive
+            badge = None
+        label = _("Note (%d/%d)") % (note_unread, note_count)
+        update('note', label, 'tryton-note', badge)
 
     def sig_switch(self, widget=None):
         if not self.modified_save():
@@ -207,7 +196,7 @@ class Form(SignalEvent, TabContent):
         current_record = self.screen.current_record
         if not current_record or current_record.id < 0:
             self.message_info(
-                _('You have to select one record.'), gtk.MESSAGE_INFO)
+                _('You have to select one record.'), Gtk.MessageType.INFO)
             return False
 
         fields = [
@@ -219,21 +208,23 @@ class Form(SignalEvent, TabContent):
         ]
 
         try:
-            res = RPCExecute('model', self.model, 'read', [current_record.id],
-                [x[0] for x in fields], context=self.screen.context)
+            data = RPCExecute('model', self.model, 'read', [current_record.id],
+                [x[0] for x in fields], context=self.screen.context)[0]
         except RPCException:
             return
         date_format = self.screen.context.get('date_format', '%x')
         datetime_format = date_format + ' %H:%M:%S.%f'
         message_str = ''
-        for line in res:
-            for (key, val) in fields:
-                value = str(line.get(key, False) or '/')
-                if line.get(key, False) \
-                        and key in ('create_date', 'write_date'):
-                    date = timezoned_date(line[key])
-                    value = common.datetime_strftime(date, datetime_format)
-                message_str += val + ' ' + value + '\n'
+        for (key, label) in fields:
+            value = data
+            keys = key.split('.')
+            name = keys.pop(-1)
+            for key in keys:
+                value = value.get(key + '.', {})
+            value = (value or {}).get(name, '/')
+            if isinstance(value, datetime.datetime):
+                value = timezoned_date(value).strftime(datetime_format)
+            message_str += '%s %s\n' % (label, value)
         message_str += _('Model:') + ' ' + self.model
         message(message_str)
         return True
@@ -276,14 +267,14 @@ class Form(SignalEvent, TabContent):
         if revision:
             format_ = self.screen.context.get('date_format', '%x')
             format_ += ' %H:%M:%S.%f'
-            revision_label = ' @ %s' % datetime_strftime(revision, format_)
+            revision_label = ' @ %s' % revision.strftime(format_)
             label = common.ellipsize(
                 self.name, 80 - len(revision_label)) + revision_label
             tooltip = self.name + revision_label
         else:
             label = common.ellipsize(self.name, 80)
             tooltip = self.name
-        self.title.set_label(label)
+        self.title.set_markup(label)
         tooltips.set_tip(self.title, tooltip)
         self.set_buttons_sensitive(revision)
 
@@ -317,15 +308,18 @@ class Form(SignalEvent, TabContent):
             msg = _('Are you sure to remove those records?')
         if sur(msg):
             if not self.screen.remove(delete=True, force_remove=True):
-                self.message_info(_('Records not removed.'), gtk.MESSAGE_ERROR)
+                self.message_info(
+                    _('Records not removed.'), Gtk.MessageType.ERROR)
             else:
-                self.message_info(_('Records removed.'), gtk.MESSAGE_INFO)
+                self.message_info(_('Records removed.'), Gtk.MessageType.INFO)
                 self.screen.count_tab_domain()
 
     def sig_import(self, widget=None):
         WinImport(self.title.get_text(), self.model, self.screen.context)
 
     def sig_export(self, widget=None):
+        if not self.modified_save():
+            return
         export = WinExport(
             self.title.get_text(), self.model,
             [r.id for r in self.screen.selected_records],
@@ -340,6 +334,26 @@ class Form(SignalEvent, TabContent):
             else:
                 export.sel_field(name)
 
+    def do_export(self, widget, export):
+        if not self.modified_save():
+            return
+        ids = [r.id for r in self.screen.selected_records]
+        fields = [f['name'] for f in export['export_fields.']]
+        data = RPCExecute(
+            'model', self.model, 'export_data', ids, fields,
+            context=self.screen.context)
+        delimiter = ','
+        if os.name == 'nt' and ',' == locale.localeconv()['decimal_point']:
+            delimiter = ';'
+        fileno, fname = tempfile.mkstemp(
+            '.csv', common.slugify(export['name']) + '_')
+        with open(fname, 'w') as fp:
+            writer = csv.writer(fp, delimiter=delimiter)
+            writer.writerow(fields)
+            writer.writerows(data)
+        os.close(fileno)
+        common.file_open(fname, 'csv')
+
     def sig_new(self, widget=None, autosave=True):
         if not common.MODELACCESS[self.model]['create']:
             return
@@ -357,7 +371,7 @@ class Form(SignalEvent, TabContent):
             return
         if self.screen.copy():
             self.message_info(_('Working now on the duplicated record(s).'),
-                gtk.MESSAGE_INFO)
+                Gtk.MessageType.INFO)
             self.screen.count_tab_domain()
 
     def sig_save(self, widget=None):
@@ -368,11 +382,12 @@ class Form(SignalEvent, TabContent):
                 or common.MODELACCESS[self.model]['create']):
             return
         if self.screen.save_current():
-            self.message_info(_('Record saved.'), gtk.MESSAGE_INFO)
+            self.message_info(_('Record saved.'), Gtk.MessageType.INFO)
             self.screen.count_tab_domain()
             return True
         else:
-            self.message_info(self.screen.invalid_message(), gtk.MESSAGE_ERROR)
+            self.message_info(
+                self.screen.invalid_message(), Gtk.MessageType.ERROR)
             return False
 
     def sig_previous(self, widget=None):
@@ -449,19 +464,20 @@ class Form(SignalEvent, TabContent):
             menu.popdown()
             return
 
-        def menu_position(menu, data):
+        def menu_position(menu, x, y, user_data):
             widget_allocation = widget.get_allocation()
-            if hasattr(widget.window, 'get_root_coords'):
-                x, y = widget.window.get_root_coords(
-                    widget_allocation.x, widget_allocation.y)
-            else:
-                x, y = widget.window.get_origin()
-                x += widget_allocation.x
-                y += widget_allocation.y
+            x, y = widget.get_window().get_root_coords(
+                widget_allocation.x, widget_allocation.y)
             return (x, y + widget_allocation.height, False)
         menu.show_all()
-        menu.popup(None, None, menu_position, 0, gtk.get_current_event_time(),
-            None)
+        if hasattr(menu, 'popup_at_widget'):
+            menu.popup_at_widget(
+                widget, Gdk.Gravity.SOUTH_WEST, Gdk.Gravity.NORTH_WEST,
+                Gtk.get_current_event())
+        else:
+            menu.popup(
+                None, None, menu_position, None, 0,
+                Gtk.get_current_event_time())
 
     def _record_message(self, screen, signal_data):
         name = '_'
@@ -484,9 +500,7 @@ class Form(SignalEvent, TabContent):
         button_switch.props.sensitive = self.screen.number_of_views > 1
 
         msg = name + ' / ' + str(signal_data[1])
-        if (signal_data[1] < signal_data[2]
-                and self.screen.limit is not None
-                and signal_data[2] > self.screen.limit):
+        if signal_data[1] < signal_data[2]:
             msg += _(' of ') + str(signal_data[2])
         self.status_label.set_text(msg)
         self.message_info()
@@ -500,7 +514,7 @@ class Form(SignalEvent, TabContent):
 
     def _record_saved(self, screen, signal_data):
         self.activate_save()
-        self.update_attachment_count()
+        self.refresh_resources()
 
     def modified_save(self):
         self.screen.save_tree_state()
@@ -512,25 +526,19 @@ class Form(SignalEvent, TabContent):
             if value == 'ok':
                 return self.sig_save(None)
             if value == 'ko':
-                record_id = self.screen.current_record.id
-                if self.sig_reload(test_modified=False):
-                    if record_id < 0:
-                        return None
-                    elif self.screen.current_record:
-                        return record_id == self.screen.current_record.id
+                return self.sig_reload(test_modified=False)
             return False
         return True
 
     def sig_close(self, widget=None):
         for dialog in reversed(self.dialogs[:]):
             dialog.destroy()
-        modified_save = self.modified_save()
-        return True if modified_save is None else modified_save
+        return self.modified_save()
 
     def _action(self, action, atype):
-        action = action.copy()
-        if self.screen.modified() and not self.sig_save():
+        if not self.modified_save():
             return
+        action = action.copy()
         record_id = (self.screen.current_record.id
             if self.screen.current_record else None)
         record_ids = [r.id for r in self.screen.selected_records]
@@ -552,10 +560,12 @@ class Form(SignalEvent, TabContent):
         gtktoolbar = super(Form, self).create_toolbar(toolbars)
 
         attach_btn = self.buttons['attach']
-        target_entry = gtk.TargetEntry.new('text/uri-list', 0, 0)
-        attach_btn.drag_dest_set(gtk.DEST_DEFAULT_ALL, [
-                target_entry,
-                ], gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY)
+        attach_btn.drag_dest_set(
+            Gtk.DestDefaults.ALL, [
+                Gtk.TargetEntry.new('text/uri-list', 0, 0),
+                Gtk.TargetEntry.new('text/plain', 0, 0),
+                ],
+            Gdk.DragAction.MOVE | Gdk.DragAction.COPY)
         attach_btn.connect('drag_data_received',
             self.attach_drag_data_received)
 
@@ -575,10 +585,10 @@ class Form(SignalEvent, TabContent):
                 ('print', 'print', _('Print'), _('Print report')),
         ):
             if action_type is not None:
-                tbutton = gtk.ToggleToolButton()
+                tbutton = Gtk.ToggleToolButton()
                 tbutton.set_icon_widget(common.IconFactory.get_image(
                         iconstock.get(special_action),
-                        gtk.ICON_SIZE_LARGE_TOOLBAR))
+                        Gtk.IconSize.LARGE_TOOLBAR))
                 tbutton.set_label(action_name)
                 tbutton._menu = self._create_popup_menu(tbutton,
                     action_type, toolbars[action_type], special_action)
@@ -589,21 +599,34 @@ class Form(SignalEvent, TabContent):
                     tbutton._can_be_sensitive = bool(
                         tbutton._menu.get_children())
             else:
-                tbutton = gtk.SeparatorToolItem()
+                tbutton = Gtk.SeparatorToolItem()
             gtktoolbar.insert(tbutton, -1)
 
-        gtktoolbar.insert(gtk.SeparatorToolItem(), -1)
+        exports = toolbars['exports']
+        if exports:
+            tbutton = self.buttons['open']
+            tbutton._can_be_sensitive = True
+            menu = tbutton._menu
+            if menu.get_children():
+                menu.add(Gtk.SeparatorMenuItem())
+            for export in exports:
+                menuitem = Gtk.MenuItem(set_underline(export['name']))
+                menuitem.set_use_underline(True)
+                menuitem.connect('activate', self.do_export, export)
+                menu.add(menuitem)
 
-        url_button = gtk.ToggleToolButton()
+        gtktoolbar.insert(Gtk.SeparatorToolItem(), -1)
+
+        url_button = Gtk.ToggleToolButton()
         url_button.set_icon_widget(
             common.IconFactory.get_image(
-                'tryton-public', gtk.ICON_SIZE_LARGE_TOOLBAR))
+                'tryton-public', Gtk.IconSize.LARGE_TOOLBAR))
         url_button.set_label(_('_Copy URL'))
         url_button.set_use_underline(True)
         self.tooltips.set_tip(
             url_button, _('Copy URL into clipboard'))
-        url_button._menu = url_menu = gtk.Menu()
-        url_menuitem = gtk.MenuItem()
+        url_button._menu = url_menu = Gtk.Menu()
+        url_menuitem = Gtk.MenuItem()
         url_menuitem.connect('activate', self.url_copy)
         url_menu.add(url_menuitem)
         url_menu.show_all()
@@ -615,7 +638,7 @@ class Form(SignalEvent, TabContent):
         return gtktoolbar
 
     def _create_popup_menu(self, widget, keyword, actions, special_action):
-        menu = gtk.Menu()
+        menu = Gtk.Menu()
         menu.connect('deactivate', self._popup_menu_hide, widget)
         widget.connect('toggled', self._update_popup_menu, menu, keyword)
 
@@ -625,10 +648,7 @@ class Form(SignalEvent, TabContent):
                 new_action['direct_print'] = True
             elif special_action == 'email':
                 new_action['email_print'] = True
-            action_name = action['name']
-            if '_' not in action_name:
-                action_name = '_' + action_name
-            menuitem = gtk.MenuItem(action_name)
+            menuitem = Gtk.MenuItem(label=set_underline(action['name']))
             menuitem.set_use_underline(True)
             menuitem.connect('activate', self._popup_menu_selected, widget,
                 new_action, keyword)
@@ -636,10 +656,10 @@ class Form(SignalEvent, TabContent):
         return menu
 
     def _popup_menu_selected(self, menuitem, togglebutton, action, keyword):
-        event = gtk.get_current_event()
+        event = Gtk.get_current_event()
         allow_similar = False
-        if (event.state & gtk.gdk.CONTROL_MASK
-                or event.state & gtk.gdk.MOD1_MASK):
+        if (event.state & Gdk.ModifierType.CONTROL_MASK
+                or event.state & Gdk.ModifierType.MOD1_MASK):
             allow_similar = True
         with Window(hide_current=True, allow_similar=allow_similar):
             self._action(action, keyword)
@@ -650,21 +670,19 @@ class Form(SignalEvent, TabContent):
 
     def _update_popup_menu(self, tbutton, menu, keyword):
         for item in menu.get_children():
-            if (getattr(item, '_update_action', False)
-                    or isinstance(item, gtk.SeparatorMenuItem)):
+            if getattr(item, '_update_action', False):
                 menu.remove(item)
 
         buttons = [b for b in self.screen.get_buttons()
             if keyword == b.attrs.get('keyword', 'action')]
         if buttons and menu.get_children():
-            menu.add(gtk.SeparatorMenuItem())
+            separator = Gtk.SeparatorMenuItem()
+            separator._update_action = True
+            menu.add(separator)
         for button in buttons:
-            menuitem = gtk.ImageMenuItem()
-            menuitem.set_label('_' + button.attrs.get('string', _('Unknown')))
-            menuitem.set_use_underline(True)
-            if button.attrs.get('icon'):
-                menuitem.set_image(common.IconFactory.get_image(
-                        button.attrs['icon'], gtk.ICON_SIZE_MENU))
+            menuitem = Gtk.MenuItem(
+                label=set_underline(button.attrs.get('string', _('Unknown'))),
+                use_underline=True)
             menuitem.connect('activate',
                 lambda m, attrs: self.screen.button(attrs), button.attrs)
             menuitem._update_action = True
@@ -683,9 +701,11 @@ class Form(SignalEvent, TabContent):
                 kw_plugins.append((name, func))
 
         if kw_plugins:
-            menu.add(gtk.SeparatorMenuItem())
+            separator = Gtk.SeparatorMenuItem()
+            separator._update_action = True
+            menu.add(separator)
         for name, func in kw_plugins:
-            menuitem = gtk.MenuItem('_' + name)
+            menuitem = Gtk.MenuItem(label=set_underline(name))
             menuitem.set_use_underline(True)
             menuitem.connect('activate', lambda m, func: func({
                         'model': self.model,
@@ -699,8 +719,11 @@ class Form(SignalEvent, TabContent):
 
     def url_copy(self, menuitem):
         url = self.screen.get_url(self.name)
-        for selection in [gtk.CLIPBOARD_PRIMARY, gtk.CLIPBOARD_CLIPBOARD]:
-            clipboard = gtk.clipboard_get(selection)
+        for selection in [
+                Gdk.Atom.intern('PRIMARY', True),
+                Gdk.Atom.intern('CLIPBOARD', True),
+                ]:
+            clipboard = Gtk.Clipboard.get(selection)
             clipboard.set_text(url, -1)
 
     def url_set(self, button, menuitem):
@@ -720,10 +743,13 @@ class Form(SignalEvent, TabContent):
         if not record or record.id < 0:
             return
         win_attach = Attachment(record,
-            lambda: self.update_attachment_count(reload=True))
+            lambda: self.refresh_resources(reload=True))
         if info == 0:
-            for uri in selection.get_uris():
-                # Win32 cut&paste terminates the list with a NULL character
-                if not uri or uri == '\0':
-                    continue
-                win_attach.add_uri(uri)
+            if selection.get_uris():
+                for uri in selection.get_uris():
+                    # Win32 cut&paste terminates the list with a NULL character
+                    if not uri or uri == '\0':
+                        continue
+                    win_attach.add_uri(uri)
+            else:
+                win_attach.add_uri(selection.get_text())
