@@ -3,7 +3,6 @@
 import os
 from itertools import chain
 import tempfile
-import logging
 import locale
 from tryton.common import \
         domain_inversion, eval_domain, localize_domain, \
@@ -16,12 +15,9 @@ import decimal
 from decimal import Decimal
 import math
 from tryton.common import RPCExecute, RPCException
-from tryton.common.htmltextbuffer import guess_decode
 from tryton.pyson import PYSONDecoder
 from tryton.config import CONFIG
 
-logger = logging.getLogger(__name__)
-
 
 class Field(object):
     '''
@@ -207,17 +203,6 @@ class CharField(Field):
     def get(self, record):
         return super(CharField, self).get(record) or self._default
 
-    def set_client(self, record, value, force_change=False):
-        if isinstance(value, bytes):
-            try:
-                value = guess_decode(value)
-            except UnicodeDecodeError:
-                logger.warning(
-                    "The encoding can not be guessed for field '%(name)s'",
-                    {'name': self.name})
-                value = None
-        super().set_client(record, value, force_change)
-
 
 class SelectionField(Field):
 
@@ -356,14 +341,14 @@ class FloatField(Field):
         value = record.value.get(self.name)
         if value is not None:
             digits = self.digits(record, factor=factor)
-            d = value * factor
-            if not isinstance(d, Decimal):
-                d = Decimal(repr(d))
             if digits:
                 p = int(digits[1])
             else:
+                d = value * factor
+                if not isinstance(d, Decimal):
+                    d = Decimal(repr(d))
                 p = -int(d.as_tuple().exponent)
-            return locale.localize('{0:.{1}f}'.format(d, p), True)
+            return locale.format('%.*f', (p, value * factor), True)
         else:
             return ''
 
@@ -428,26 +413,27 @@ class M2OField(Field):
         return self.get(record) is None
 
     def get_client(self, record):
-        rec_name = record.value.get(self.name + '.rec_name')
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name')
         if rec_name is None:
             self.set(record, self.get(record))
-            rec_name = record.value.get(self.name + '.rec_name') or ''
-        return rec_name
+            rec_name = record.value.get(self.name + '.', {}).get('rec_name')
+        return rec_name or ''
 
     def set_client(self, record, value, force_change=False):
         if isinstance(value, (tuple, list)):
             value, rec_name = value
         else:
             if value == self.get(record):
-                rec_name = record.value.get(self.name + '.rec_name', '')
+                rec_name = record.value.get(
+                    self.name + '.', {}).get('rec_name', '')
             else:
                 rec_name = ''
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
         super(M2OField, self).set_client(record, value,
             force_change=force_change)
 
     def set(self, record, value):
-        rec_name = record.value.get(self.name + '.rec_name') or ''
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name') or ''
         if not rec_name and value is not None and value >= 0:
             try:
                 result, = RPCExecute('model', self.attrs['relation'], 'read',
@@ -455,7 +441,7 @@ class M2OField(Field):
             except RPCException:
                 return
             rec_name = result['rec_name'] or ''
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
         record.value[self.name] = value
 
     def get_context(self, record, record_context=None, local=False):
@@ -637,10 +623,10 @@ class O2MField(Field):
                 group.load_fields(fields)
 
         if mode == 'list ids':
-            records_to_remove = [r for r in group if r.id not in value]
-            for record_to_remove in records_to_remove:
-                group.remove(record_to_remove, remove=True, signal=False)
-            group.load(value, modified=modified or default)
+            for old_record in group:
+                if old_record.id not in value:
+                    group.remove(old_record, remove=True, signal=False)
+            group.load(value, modified=modified)
         else:
             for vals in value:
                 new_record = record.value[self.name].new(default=False)
@@ -814,7 +800,7 @@ class ReferenceField(Field):
     def get_client(self, record):
         if record.value.get(self.name):
             model, _ = record.value[self.name]
-            name = record.value.get(self.name + '.rec_name') or ''
+            name = record.value.get(self.name + '.', {}).get('rec_name') or ''
             return model, name
         else:
             return None
@@ -842,10 +828,10 @@ class ReferenceField(Field):
                         pass
                 if '%s,%s' % (ref_model, ref_id) == self.get(record):
                     rec_name = record.value.get(
-                        self.name + '.rec_name') or ''
+                        self.name + '.', {}).get('rec_name', '')
                 else:
                     rec_name = ''
-            record.value[self.name + '.rec_name'] = rec_name
+            record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
             value = (ref_model, ref_id)
         super(ReferenceField, self).set_client(record, value,
             force_change=force_change)
@@ -865,7 +851,7 @@ class ReferenceField(Field):
                     pass
         else:
             ref_model, ref_id = value
-        rec_name = record.value.get(self.name + '.rec_name') or ''
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name') or ''
         if ref_model and ref_id is not None and ref_id >= 0:
             if not rec_name and ref_id >= 0:
                 try:
@@ -877,9 +863,9 @@ class ReferenceField(Field):
         elif ref_model:
             rec_name = ''
         else:
-            rec_name = str(ref_id) if ref_id is not None else ''
+            rec_name = str(ref_id)
         record.value[self.name] = ref_model, ref_id
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
 
     def get_context(self, record, record_context=None, local=False):
         context = super(ReferenceField, self).get_context(
@@ -905,14 +891,13 @@ class ReferenceField(Field):
         else:
             model = None
         screen_domain, attr_domain = self.domains_get(record)
-        screen_domain = filter_leaf(screen_domain, self.name, model)
         screen_domain = prepare_reference_domain(screen_domain, self.name)
         return concat(localize_domain(
-                screen_domain, self.name, strip_target=True), attr_domain)
+                filter_leaf(screen_domain, self.name, model),
+                strip_target=True), attr_domain)
 
     def get_models(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        screen_domain = prepare_reference_domain(screen_domain, self.name)
         return extract_reference_models(
             concat(screen_domain, attr_domain), self.name)
 
@@ -921,6 +906,12 @@ class _FileCache(object):
     def __init__(self, path):
         self.path = path
 
+    def __del__(self):
+        try:
+            os.remove(self.path)
+        except IOError:
+            pass
+
 
 class BinaryField(Field):
 
