@@ -91,7 +91,6 @@ class Group(SignalEvent, list):
 
     def insert(self, pos, record):
         assert record.group is self
-        pos = min(pos, len(self))
         if pos >= 1:
             self.__getitem__(pos - 1).next[id(self)] = record
         if pos < self.__len__():
@@ -101,7 +100,7 @@ class Group(SignalEvent, list):
         super(Group, self).insert(pos, record)
         self.__id2record[record.id] = record
         if not self.lock_signal:
-            self.signal('group-list-changed', ('record-added', record))
+            self.signal('group-list-changed', ('record-added', record, pos))
 
     def append(self, record):
         assert record.group is self
@@ -111,7 +110,8 @@ class Group(SignalEvent, list):
         super(Group, self).append(record)
         self.__id2record[record.id] = record
         if not self.lock_signal:
-            self.signal('group-list-changed', ('record-added', record))
+            self.signal('group-list-changed', (
+                    'record-added', record, self.__len__() - 1))
 
     def _remove(self, record):
         idx = self.index(record)
@@ -121,19 +121,20 @@ class Group(SignalEvent, list):
                     self.__getitem__(idx + 1)
             else:
                 self.__getitem__(idx - 1).next[id(self)] = None
-        self.signal('group-list-changed', ('record-removed', record))
+        self.signal('group-list-changed', ('record-removed', record, idx))
         super(Group, self).remove(record)
         del self.__id2record[record.id]
 
     def clear(self):
         # Use reversed order to minimize the cursor reposition as the cursor
         # has more chances to be on top of the list.
+        length = self.__len__()
         for record in reversed(self[:]):
-            # Destroy record before propagating the signal to recursively
-            # destroy also the underlying records
+            self.signal(
+                'group-list-changed', ('record-removed', record, length - 1))
             record.destroy()
-            self.signal('group-list-changed', ('record-removed', record))
             self.pop()
+            length -= 1
         self.__id2record = {}
         self.record_removed, self.record_deleted = [], []
 
@@ -185,7 +186,6 @@ class Group(SignalEvent, list):
         ctx['_timestamp'] = {}
         for rec in records:
             ctx['_timestamp'].update(rec.get_timestamp())
-            rec.destroy()
         record_ids = set(r.id for r in records)
         reload_ids = set(root_group.on_write_ids(list(record_ids)))
         reload_ids -= record_ids
@@ -396,25 +396,23 @@ class Group(SignalEvent, list):
     def remove(self, record, remove=False, modified=True, signal=True,
             force_remove=False):
         idx = self.index(record)
-        if record.id >= 0:
+        if self[idx].id >= 0:
             if remove:
-                if record in self.record_deleted:
-                    self.record_deleted.remove(record)
-                if record not in self.record_removed:
-                    self.record_removed.append(record)
+                if self[idx] in self.record_deleted:
+                    self.record_deleted.remove(self[idx])
+                self.record_removed.append(self[idx])
             else:
-                if record in self.record_removed:
-                    self.record_removed.remove(record)
-                if record not in self.record_deleted:
-                    self.record_deleted.append(record)
+                if self[idx] in self.record_removed:
+                    self.record_removed.remove(self[idx])
+                self.record_deleted.append(self[idx])
         if record.parent:
             record.parent.modified_fields.setdefault('id')
             record.parent.signal('record-modified')
         if modified:
             record.modified_fields.setdefault('id')
             record.signal('record-modified')
-        if record.id < 0 or force_remove:
-            self._remove(record)
+        if self[idx].id < 0 or force_remove:
+            self._remove(self[idx])
 
         if len(self):
             self.current_idx = min(idx, len(self) - 1)
