@@ -3,7 +3,6 @@
 "Screen"
 import copy
 import functools
-import gobject
 import datetime
 import calendar
 import json
@@ -14,7 +13,7 @@ import gettext
 import logging
 from operator import itemgetter
 
-import gtk
+from gi.repository import GLib, Gtk
 
 from tryton.gui.window.view_form.model.group import Group
 from tryton.gui.window.view_form.view.screen_container import ScreenContainer
@@ -27,6 +26,7 @@ from tryton.common import RPCExecute, RPCException, MODELACCESS, \
     node_attributes, sur, RPCContextReload, warning
 from tryton.action import Action
 from tryton.pyson import PYSONDecoder
+from tryton.rpc import clear_cache
 
 _ = gettext.gettext
 logger = logging.getLogger(__name__)
@@ -93,19 +93,19 @@ class Screen(SignalEvent):
                         yield widget
 
             for widget in reversed(list(walk_descendants(context_widget))):
-                if isinstance(widget, gtk.Entry):
+                if isinstance(widget, Gtk.Entry):
                     widget.connect_after(
                         'activate', self.screen_container.activate)
-                elif isinstance(widget, gtk.CheckButton):
+                elif isinstance(widget, Gtk.CheckButton):
                     widget.connect_after(
                         'toggled', self.screen_container.activate)
 
             def remove_bin(widget):
-                assert isinstance(widget, (gtk.ScrolledWindow, gtk.Viewport))
+                assert isinstance(widget, (Gtk.ScrolledWindow, Gtk.Viewport))
                 parent = widget.get_parent()
                 parent.remove(widget)
                 child = widget.get_child()
-                while isinstance(child, (gtk.ScrolledWindow, gtk.Viewport)):
+                while isinstance(child, (Gtk.ScrolledWindow, Gtk.Viewport)):
                     child = child.get_child()
                 child.get_parent().remove(child)
                 parent.add(child)
@@ -119,7 +119,7 @@ class Screen(SignalEvent):
                 remove_bin(widget)
 
             self.screen_container.filter_vbox.pack_start(
-                context_widget, expand=False, fill=True)
+                context_widget, expand=False, fill=True, padding=0)
             self.screen_container.filter_vbox.reorder_child(
                 context_widget, 0)
             self.context_screen.widget.show()
@@ -238,7 +238,7 @@ class Screen(SignalEvent):
 
     def search_prev(self, search_string):
         if self.limit:
-            self.offset -= self.limit
+            self.offset = max(self.offset - self.limit, 0)
         self.search_filter(search_string=search_string)
 
     def search_next(self, search_string):
@@ -257,9 +257,7 @@ class Screen(SignalEvent):
                 self.context_screen.display(set_cursor=True)
                 return False
             context = self.local_context
-            screen_context = self.context_screen.get_on_change_value()
-            screen_context.pop('id')
-            context.update(screen_context)
+            context.update(self.context_screen.get_on_change_value())
             self.new_group(context)
 
         domain = self.search_domain(search_string, True)
@@ -273,11 +271,16 @@ class Screen(SignalEvent):
         context = self.context
         if self.screen_container.but_active.get_active():
             context['active_test'] = False
-        try:
-            ids = RPCExecute('model', self.model_name, 'search', domain,
-                self.offset, self.limit, self.order, context=context)
-        except RPCException:
-            ids = []
+        ids = []
+        while True:
+            try:
+                ids = RPCExecute('model', self.model_name, 'search', domain,
+                    self.offset, self.limit, self.order, context=context)
+            except RPCException:
+                break
+            if ids or self.offset <= 0:
+                break
+            self.offset = max(self.offset - self.limit, 0)
         if not only_ids:
             if self.limit is not None and len(ids) == self.limit:
                 try:
@@ -378,6 +381,7 @@ class Screen(SignalEvent):
                 fields[name] = field.attrs
                 fields_views[name] = field.views
         self.tree_states_done.clear()
+        self.order = None
         self.__group = group
         self.parent = group.parent
         self.parent_name = group.parent_name
@@ -441,39 +445,21 @@ class Screen(SignalEvent):
             pos = 0
         self.signal('record-message', (pos, len(self.group) + self.offset,
             self.search_count, record and record.id))
-        attachment_count = 0
-        if record and record.attachment_count > 0:
-            attachment_count = record.attachment_count
-        self.signal('attachment-count', attachment_count)
-        unread_note = 0
-        if record and record.unread_note > 0:
-            unread_note = record.unread_note
-        self.signal('unread-note', unread_note)
-        # update attachment-count after 1 second
-        gobject.timeout_add(1000, self.update_attachment, record)
-        gobject.timeout_add(1000, self.update_note, record)
+        self.signal('resources', record.resources if record else None)
+        # update resources after 1 second
+        GLib.timeout_add(1000, self._update_resources, record)
         return True
 
     current_record = property(__get_current_record, __set_current_record)
 
-    def update_attachment(self, record):
-        if record != self.current_record:
-            return False
-        if record and self.signal_connected('attachment-count'):
-            attachment_count = record.get_attachment_count()
-            self.signal('attachment-count', attachment_count)
-        return False
-
-    def update_note(self, record):
-        if record != self.current_record:
-            return False
-        if record and self.signal_connected('unread-note'):
-            unread_note = record.get_unread_note()
-            self.signal('unread-note', unread_note)
+    def _update_resources(self, record):
+        if (record
+                and record == self.current_record
+                and self.signal_connected('resources')):
+            self.signal('resources', record.get_resources())
         return False
 
     def destroy(self):
-        self.screen_container.destroy()
         for view in self.views:
             view.destroy()
         del self.views[:]
@@ -542,7 +528,7 @@ class Screen(SignalEvent):
         self.screen_container.set(self.current_view.widget)
         self.display()
         # Postpone set of the cursor to ensure widgets are allocated
-        gobject.idle_add(self.set_cursor)
+        GLib.idle_add(self.set_cursor)
 
     def load_view_to_load(self):
         if len(self.view_to_load):
@@ -591,8 +577,8 @@ class Screen(SignalEvent):
         self.group.add_fields(fields)
         for field in fields:
             self.group.fields[field].views.add(view_id)
-        view = View.parse(self, xml_dom, view.get('field_childs'))
-        view.view_id = view_id
+        view = View.parse(
+            self, view_id, view['type'], xml_dom, view.get('field_childs'))
         self.views.append(view)
 
         return view
@@ -616,7 +602,7 @@ class Screen(SignalEvent):
         self.current_record = record
         self.display()
         # Postpone set of the cursor to ensure widgets are allocated
-        gobject.idle_add(self.set_cursor, True)
+        GLib.idle_add(self.set_cursor, True)
         return self.current_record
 
     def new_model_position(self):
@@ -680,7 +666,7 @@ class Screen(SignalEvent):
         current_view = self.current_view
         if not current_view:
             return
-        elif current_view.view_type in ('tree', 'form'):
+        elif current_view.view_type in ('tree', 'form', 'list-form'):
             current_view.set_cursor(new=new, reset_view=reset_view)
 
     def get(self):
@@ -796,13 +782,9 @@ class Screen(SignalEvent):
         if parent is not None and parent < 0:
             return
         expanded_nodes, selected_nodes = [], []
-        timestamp = self.parent._timestamp if self.parent else None
         state = self.tree_states[parent][view.children_field]
         if state:
-            state_timestamp, expanded_nodes, selected_nodes = state
-            if (timestamp != state_timestamp
-                    and view.view_type != 'form'):
-                state = None
+            expanded_nodes, selected_nodes = state
         if state is None and CONFIG['client.save_tree_state']:
             json_domain = self.get_tree_domain(parent)
             try:
@@ -817,7 +799,7 @@ class Screen(SignalEvent):
                     _('Unable to get view tree state for %s')
                     % self.model_name)
             self.tree_states[parent][view.children_field] = (
-                timestamp, expanded_nodes, selected_nodes)
+                expanded_nodes, selected_nodes)
         if view.view_type == 'tree':
             view.expand_nodes(expanded_nodes)
             view.select_nodes(selected_nodes)
@@ -842,7 +824,6 @@ class Screen(SignalEvent):
 
     def save_tree_state(self, store=True):
         parent = self.parent.id if self.parent else None
-        timestamp = self.parent._timestamp if self.parent else None
         for view in self.views:
             if view.view_type == 'form':
                 for widgets in view.widgets.values():
@@ -855,12 +836,12 @@ class Screen(SignalEvent):
                         path = -self.current_record.group.index(
                             self.current_record)
                     self.tree_states[parent][view.children_field] = (
-                        timestamp, [], [[path]])
+                        [], [[path]])
             elif view.view_type == 'tree':
                 paths = view.get_expanded_paths()
                 selected_paths = view.get_selected_paths()
                 self.tree_states[parent][view.children_field] = (
-                    timestamp, paths, selected_paths)
+                    paths, selected_paths)
                 if (store
                         and int(view.attributes.get('tree_state', False))
                         and CONFIG['client.save_tree_state']):
@@ -873,6 +854,7 @@ class Screen(SignalEvent):
                             self.model_name, json_domain, view.children_field,
                             json_paths, json_selected_path,
                             process_exception=False)
+                        clear_cache('model.ir.ui.view_tree_state.get')
                     except Exception:
                         logger.warn(
                             _('Unable to set view tree state'), exc_info=True)
@@ -887,6 +869,8 @@ class Screen(SignalEvent):
         return json_domain
 
     def load(self, ids, set_cursor=True, modified=False):
+        self.tree_states.clear()
+        self.tree_states_done.clear()
         self.group.load(ids, modified=modified)
         self.current_view.reset()
         if ids and self.current_view.view_type != 'calendar':
@@ -934,16 +918,14 @@ class Screen(SignalEvent):
         view.set_value()
         self.set_cursor(reset_view=False)
         if view.view_type == 'tree' and len(self.group):
-            range_ = view.treeview.get_visible_range()
-            if range_:
-                start, end = range_
-                vadjustment = view.treeview.get_vadjustment()
-                vadjustment.props.value = min(
-                    vadjustment.props.value + vadjustment.props.page_increment,
-                    vadjustment.props.upper)
-                model = view.treeview.get_model()
-                iter_ = model.get_iter(end)
-                self.current_record = model.get_value(iter_, 0)
+            start, end = view.treeview.get_visible_range()
+            vadjustment = view.treeview.get_vadjustment()
+            vadjustment.props.value = min(
+                vadjustment.props.value + vadjustment.props.page_increment,
+                vadjustment.props.upper)
+            model = view.treeview.get_model()
+            iter_ = model.get_iter(end)
+            self.current_record = model.get_value(iter_, 0)
         elif (view.view_type == 'form'
                 and self.current_record
                 and self.current_record.group):
@@ -972,6 +954,11 @@ class Screen(SignalEvent):
                 record = next
                 break
             self.current_record = record
+        elif (view.view_type == 'list-form' and len(self.group)
+                and self.current_record in self.group):
+            idx = self.group.index(self.current_record)
+            if 0 <= idx < len(self.group) - 1:
+                self.current_record = self.group[idx + 1]
         elif view.view_type == 'calendar':
             record = self.current_record
             goocalendar = view.widgets.get('goocalendar')
@@ -1007,16 +994,14 @@ class Screen(SignalEvent):
         view.set_value()
         self.set_cursor(reset_view=False)
         if view.view_type == 'tree' and len(self.group):
-            range_ = view.treeview.get_visible_range()
-            if range_:
-                start, end = range_
-                vadjustment = view.treeview.get_vadjustment()
-                vadjustment.props.value = min(
-                    vadjustment.props.value - vadjustment.props.page_increment,
-                    vadjustment.props.lower)
-                model = view.treeview.get_model()
-                iter_ = model.get_iter(start)
-                self.current_record = model.get_value(iter_, 0)
+            start, end = view.treeview.get_visible_range()
+            vadjustment = view.treeview.get_vadjustment()
+            vadjustment.props.value = min(
+                vadjustment.props.value - vadjustment.props.page_increment,
+                vadjustment.props.lower)
+            model = view.treeview.get_model()
+            iter_ = model.get_iter(start)
+            self.current_record = model.get_value(iter_, 0)
         elif (view.view_type == 'form'
                 and self.current_record
                 and self.current_record.group):
@@ -1060,6 +1045,11 @@ class Screen(SignalEvent):
                             if prev_id >= 0:
                                 self.current_record = events[prev_id].record
                             break
+        elif (view.view_type == 'list-form' and len(self.group)
+                and self.current_record in self.group):
+            idx = self.group.index(self.current_record)
+            if 0 < idx <= len(self.group) - 1:
+                self.current_record = self.group[idx - 1]
         else:
             self.current_record = self.group[-1] if len(self.group) else None
         self.set_cursor(reset_view=False)
@@ -1096,9 +1086,6 @@ class Screen(SignalEvent):
     def clear(self):
         self.current_record = None
         self.group.clear()
-        self.tree_states_done.clear()
-        for view in self.views:
-            view.reset()
 
     def on_change(self, fieldname, attr):
         self.current_record.on_change(fieldname, attr)
@@ -1229,6 +1216,10 @@ class Screen(SignalEvent):
         if name:
             query_string.append(
                 ('name', json.dumps(name, separators=(',', ':'))))
+        if self.screen_container.tab_domain:
+            query_string.append(('tab_domain', json.dumps(
+                        self.screen_container.tab_domain,
+                        cls=JSONEncoder, separators=(',', ':'))))
         path = [CONFIG['login.db'], 'model', self.model_name]
         view_ids = [v.view_id for v in self.views] + self.view_ids
         if self.current_view.view_type != 'form':
