@@ -3,7 +3,7 @@
 from gettext import gettext as _
 from weakref import WeakKeyDictionary
 
-import gtk
+from gi.repository import Gtk, Gdk
 
 from .textbox import TextBox
 from tryton.common import get_toplevel_window, IconFactory
@@ -25,71 +25,81 @@ class RichTextBox(TextBox):
         self.colors = {}
         if int(self.attrs.get('toolbar', 1)):
             self.toolbar = self.get_toolbar(self.textview)
-            self.widget.pack_start(self.toolbar, expand=False, fill=True)
+            self.widget.pack_start(
+                self.toolbar, expand=False, fill=True, padding=0)
 
     def get_toolbar(self, textview):
-        toolbar = gtk.Toolbar()
+        toolbar = Gtk.Toolbar()
         toolbar.set_style({
                 'default': False,
-                'both': gtk.TOOLBAR_BOTH,
-                'text': gtk.TOOLBAR_TEXT,
-                'icons': gtk.TOOLBAR_ICONS}[CONFIG['client.toolbar']])
+                'both': Gtk.ToolbarStyle.BOTH,
+                'text': Gtk.ToolbarStyle.TEXT,
+                'icons': Gtk.ToolbarStyle.ICONS,
+                }[CONFIG['client.toolbar']])
         tag_widgets = self.tag_widgets[textview] = {}
 
-        for icon in ['bold', 'italic', 'underline']:
-            button = gtk.ToggleToolButton()
+        for icon, label in [
+                ('bold', _("Bold")),
+                ('italic', _("Italic")),
+                ('underline', _("Underline")),
+                ]:
+            button = Gtk.ToggleToolButton()
             button.set_icon_widget(IconFactory.get_image(
                     'tryton-format-%s' % icon,
-                    gtk.ICON_SIZE_SMALL_TOOLBAR))
+                    Gtk.IconSize.SMALL_TOOLBAR))
+            button.set_label(label)
             button.connect('toggled', self.toggle_props, icon, textview)
             toolbar.insert(button, -1)
             tag_widgets[icon] = button
 
-        toolbar.insert(gtk.SeparatorToolItem(), -1)
+        toolbar.insert(Gtk.SeparatorToolItem(), -1)
 
         for name, options, active in [
                 ('family', FAMILIES, FAMILIES.index('normal')),
                 ('size', SIZES, SIZES.index('4')),
                 ]:
-            try:
-                combobox = gtk.ComboBoxText()
-            except AttributeError:
-                combobox = gtk.combo_box_new_text()
+            combobox = Gtk.ComboBoxText()
             for option in options:
                 combobox.append_text(option)
             combobox.set_active(active)
             combobox.set_focus_on_click(False)
             combobox.connect('changed', self.change_props, name, textview)
-            tool = gtk.ToolItem()
+            tool = Gtk.ToolItem()
             tool.add(combobox)
             toolbar.insert(tool, -1)
             tag_widgets[name] = combobox
 
-        toolbar.insert(gtk.SeparatorToolItem(), -1)
+        toolbar.insert(Gtk.SeparatorToolItem(), -1)
 
         button = None
-        for name in ['left', 'center', 'right', 'justify']:
+        for name, label in [
+                ('left', _("Align Left")),
+                ('center', _("Align Center")),
+                ('right', _("Align Right")),
+                ('justify', _("Justify")),
+                ]:
             icon = 'tryton-format-align-%s' % name
-            button = gtk.RadioToolButton.new_from_widget(button)
+            button = Gtk.RadioToolButton.new_from_widget(button)
             button.set_icon_widget(IconFactory.get_image(
-                    icon, gtk.ICON_SIZE_SMALL_TOOLBAR))
+                    icon, Gtk.IconSize.SMALL_TOOLBAR))
             button.set_active(icon == 'left')
+            button.set_label(label)
             button.connect(
                 'toggled', self.toggle_justification, name, textview)
             toolbar.insert(button, -1)
             tag_widgets[name] = button
 
-        toolbar.insert(gtk.SeparatorToolItem(), -1)
+        toolbar.insert(Gtk.SeparatorToolItem(), -1)
 
         for icon, label in [
-                ('foreground', _('Foreground')),
+                ('foreground', _("Foreground Color")),
                 # TODO ('background', _('Background')),
                 ]:
-            button = gtk.ToolButton()
+            button = Gtk.ToolButton()
             if icon == 'foreground':
                 button.set_icon_widget(IconFactory.get_image(
                         'tryton-format-color-text',
-                        gtk.ICON_SIZE_SMALL_TOOLBAR))
+                        Gtk.IconSize.SMALL_TOOLBAR))
             button.set_label(label)
             button.connect('clicked', self.toggle_color, icon, textview)
             toolbar.insert(button, -1)
@@ -114,7 +124,7 @@ class RichTextBox(TextBox):
         textview = widget.get_children()[-1].get_child()
         if self.toolbar:
             widget.pack_start(
-                self.get_toolbar(textview), expand=False, fill=True)
+                self.get_toolbar(textview), expand=False, fill=True, padding=0)
         return widget
 
     def translate_widget_set_readonly(self, widget, value):
@@ -125,13 +135,13 @@ class RichTextBox(TextBox):
                 tool = toolbar.get_nth_item(n)
                 tool.set_sensitive(not value)
 
-    def set_value(self, record, field):
+    def set_value(self):
         # avoid modification of not normalized value
         value = self.get_value()
-        prev_value = field.get_client(record) or ''
+        prev_value = self.field.get_client(self.record) or ''
         if value == normalize_markup(prev_value):
             value = prev_value
-        field.set_client(record, value)
+        self.field.set_client(self.record, value)
 
     @property
     def modified(self):
@@ -169,8 +179,6 @@ class RichTextBox(TextBox):
             self.toolbar.set_sensitive(not value)
 
     def detect_style(self, textview, *args):
-        if not self.toolbar:
-            return
         tag_widgets = self.tag_widgets[textview]
         text_buffer = textview.get_buffer()
         try:
@@ -210,6 +218,7 @@ class RichTextBox(TextBox):
 
         bolds, italics, underlines = set(), set(), set()
         families, sizes, justifications = set(), set(), set()
+        self.colors['foreground'] = 'black'
 
         iter_ = start.copy()
         while True:
@@ -235,6 +244,8 @@ class RichTextBox(TextBox):
                     size = SIZES.index(size)
                 elif tag.props.name.startswith('justification'):
                     _, justification = tag.props.name.split()
+                elif tag.props.name.startswith('foreground'):
+                    _, self.colors['foreground'] = tag.props.name.split()
             bolds.add(bold)
             italics.add(italic)
             underlines.add(underline)
@@ -258,8 +269,6 @@ class RichTextBox(TextBox):
         toggle_justification(justifications, True)
 
     def insert_text_style(self, text_buffer, iter_, text, length, textview):
-        if not self.toolbar:
-            return
         # Text is already inserted so iter_ point to the end
         start = iter_.copy()
         start.backward_chars(length)
@@ -270,19 +279,19 @@ class RichTextBox(TextBox):
 
     def _apply_tool(self, text_buffer, name, tool, start, end):
         # First test RadioToolButton as they inherit from ToggleToolButton
-        if isinstance(tool, gtk.RadioToolButton):
+        if isinstance(tool, Gtk.RadioToolButton):
             name = 'justification %s' % name
             if not tool.get_active():
                 remove_tags(text_buffer, start, end, name)
             else:
                 remove_tags(text_buffer, start, end, 'justification')
                 text_buffer.apply_tag_by_name(name, start, end)
-        elif isinstance(tool, gtk.ToggleToolButton):
+        elif isinstance(tool, Gtk.ToggleToolButton):
             if tool.get_active():
                 text_buffer.apply_tag_by_name(name, start, end)
             else:
                 text_buffer.remove_tag_by_name(name, start, end)
-        elif isinstance(tool, gtk.ComboBox):
+        elif isinstance(tool, Gtk.ComboBoxText):
             value = tool.get_active_text()
             remove_tags(text_buffer, start, end, name)
             name = '%s %s' % (name, value)
@@ -329,16 +338,16 @@ class RichTextBox(TextBox):
             start = start.get_offset()
             end = end.get_offset()
 
-        dialog = gtk.ColorSelectionDialog(_('Select a color'))
-        dialog.set_transient_for(get_toplevel_window())
-        colorsel = dialog.get_color_selection()
-        colorsel.set_has_palette(True)
-        color = self.colors.get(name)
-        if color:
-            colorsel.set_current_rgba(color)
-        if dialog.run() == gtk.RESPONSE_OK:
-            color = colorsel.get_current_rgba()
-            self.colors[name] = color
+        dialog = Gtk.ColorChooserDialog(
+            title=_('Select a color'),
+            transient_for=get_toplevel_window(),
+            use_alpha=False)
+        color = Gdk.RGBA()
+        if name in self.colors:
+            color.parse(self.colors[name])
+            dialog.set_rgba(color)
+        if dialog.run() == Gtk.ResponseType.OK:
+            color = dialog.get_rgba()
             if start is not None and end is not None:
                 start = text_buffer.get_iter_at_offset(start)
                 end = text_buffer.get_iter_at_offset(end)
